@@ -7,12 +7,19 @@ See http://dev.chromium.org/developers/how-tos/depottools/presubmit-scripts
 for more details on the presubmit API built into depot_tools.
 """
 
+import itertools
 import os
 import re
 import shutil
 import subprocess
 import sys
 import tempfile
+import textwrap
+import pathlib
+
+# This line is 'magic' in that git-cl looks for it to decide whether to
+# use Python3 instead of Python2 when running the code in this file.
+USE_PYTHON3 = True
 
 # Fragment of a regular expression that matches C/C++ and Objective-C++ implementation files and headers.
 _IMPLEMENTATION_AND_HEADER_EXTENSIONS = r'\.(c|cc|cpp|cxx|mm|h|hpp|hxx)$'
@@ -28,6 +35,20 @@ _PRIMARY_EXPORT_TARGETS = [
 ]
 
 
+def _SplitIntoMultipleCommits(description_text):
+    paragraph_split_pattern = r"(?m)(^\s*$\n)"
+    multiple_paragraphs = re.split(paragraph_split_pattern, description_text)
+    multiple_commits = [""]
+    change_id_pattern = re.compile(r"(?m)^Change-Id: [a-zA-Z0-9]*$")
+    for paragraph in multiple_paragraphs:
+        multiple_commits[-1] += paragraph
+        if change_id_pattern.search(paragraph):
+            multiple_commits.append("")
+    if multiple_commits[-1] == "":
+        multiple_commits.pop()
+    return multiple_commits
+
+
 def _CheckCommitMessageFormatting(input_api, output_api):
 
     def _IsLineBlank(line):
@@ -44,23 +65,10 @@ def _CheckCommitMessageFormatting(input_api, output_api):
     def _IsTagLine(line):
         return ":" in line
 
-    def _SplitIntoMultipleCommits(description_text):
-        paragraph_split_pattern = r"((?m)^\s*$\n)"
-        multiple_paragraphs = re.split(paragraph_split_pattern, description_text)
-        multiple_commits = [""]
-        change_id_pattern = re.compile(r"(?m)^Change-Id: [a-zA-Z0-9]*$")
-        for paragraph in multiple_paragraphs:
-            multiple_commits[-1] += paragraph
-            if change_id_pattern.search(paragraph):
-                multiple_commits.append("")
-        if multiple_commits[-1] == "":
-            multiple_commits.pop()
-        return multiple_commits
-
     def _CheckTabInCommit(lines):
         return all([line.find("\t") == -1 for line in lines])
 
-    allowlist_strings = ['Revert "', 'Roll ', 'Reland ', 'Re-land ']
+    allowlist_strings = ['Revert', 'Roll', 'Manual roll', 'Reland', 'Re-land']
     summary_linelength_warning_lower_limit = 65
     summary_linelength_warning_upper_limit = 70
     description_linelength_limit = 72
@@ -162,9 +170,9 @@ def _CheckCommitMessageFormatting(input_api, output_api):
         # loop through description body
         while len(commit_msg_lines) > 0:
             line = commit_msg_lines.pop(0)
-            # lines starting with 4 spaces or lines without space(urls)
+            # lines starting with 4 spaces, quotes or lines without space(urls)
             # are exempt from length check
-            if line.startswith("    ") or " " not in line:
+            if line.startswith("    ") or line.startswith("> ") or " " not in line:
                 continue
             if len(line) > description_linelength_limit:
                 errors.append(
@@ -180,12 +188,6 @@ def _CheckCommitMessageFormatting(input_api, output_api):
 def _CheckChangeHasBugField(input_api, output_api):
     """Requires that the changelist have a Bug: field from a known project."""
     bugs = input_api.change.BugsFromDescription()
-    if not bugs:
-        return [
-            output_api.PresubmitError('Please ensure that your description contains:\n'
-                                      '"Bug: angleproject:[bug number]"\n'
-                                      'directly above the Change-Id tag.')
-        ]
 
     # The bug must be in the form of "project:number".  None is also accepted, which is used by
     # rollers as well as in very minor changes.
@@ -197,27 +199,35 @@ def _CheckChangeHasBugField(input_api, output_api):
     ]
     bug_regex = re.compile(r"([a-z]+[:/])(\d+)")
     errors = []
-    extra_help = None
+    extra_help = False
+
+    if not bugs:
+        errors.append('Please ensure that your description contains\n'
+                      'Bug: bugtag\n'
+                      'directly above the Change-Id tag (no empty line in-between)')
+        extra_help = True
 
     for bug in bugs:
         if bug == 'None':
-            errors.append(
-                output_api.PresubmitError('Invalid bug tag "None" in presence of other bug tags.'))
+            errors.append('Invalid bug tag "None" in presence of other bug tags.')
             continue
 
         match = re.match(bug_regex, bug)
         if match == None or bug != match.group(0) or match.group(1) not in projects:
-            errors.append(output_api.PresubmitError('Incorrect bug tag "' + bug + '".'))
-            if not extra_help:
-                extra_help = output_api.PresubmitError('Acceptable format is:\n\n'
-                                                       '    Bug: project:bugnumber\n\n'
-                                                       'Acceptable projects are:\n\n    ' +
-                                                       '\n    '.join(projects))
+            errors.append('Incorrect bug tag "' + bug + '".')
+            extra_help = True
 
     if extra_help:
-        errors.append(extra_help)
+        change_ids = re.findall('^Change-Id:', input_api.change.FullDescriptionText(), re.M)
+        if len(change_ids) > 1:
+            errors.append('Note: multiple Change-Id tags found in description')
 
-    return errors
+        errors.append('''Acceptable bugtags:
+    project:bugnumber - where project is one of ({projects})
+    b/bugnumber - for Buganizer/IssueTracker bugs
+'''.format(projects=', '.join(p[:-1] for p in projects if p != 'b/')))
+
+    return [output_api.PresubmitError('\n\n'.join(errors))] if errors else []
 
 
 def _CheckCodeGeneration(input_api, output_api):
@@ -225,7 +235,7 @@ def _CheckCodeGeneration(input_api, output_api):
     class Msg(output_api.PresubmitError):
         """Specialized error message"""
 
-        def __init__(self, message):
+        def __init__(self, message, **kwargs):
             super(output_api.PresubmitError, self).__init__(
                 message,
                 long_text='Please ensure your ANGLE repositiory is synced to tip-of-tree\n'
@@ -234,12 +244,13 @@ def _CheckCodeGeneration(input_api, output_api):
                 'If that fails, run scripts/run_code_generation.py to refresh generated hashes.\n'
                 '\n'
                 'If you are building ANGLE inside Chromium you must bootstrap ANGLE\n'
-                'before gclient sync. See the DevSetup documentation for more details.\n')
+                'before gclient sync. See the DevSetup documentation for more details.\n',
+                **kwargs)
 
     code_gen_path = input_api.os_path.join(input_api.PresubmitLocalPath(),
                                            'scripts/run_code_generation.py')
     cmd_name = 'run_code_generation'
-    cmd = [input_api.python_executable, code_gen_path, '--verify-no-dirty']
+    cmd = [input_api.python3_executable, code_gen_path, '--verify-no-dirty']
     test_cmd = input_api.Command(name=cmd_name, cmd=cmd, kwargs={}, message=Msg)
     if input_api.verbose:
         print('Running ' + cmd_name)
@@ -296,11 +307,12 @@ def _CheckExportValidity(input_api, output_api):
     use_shell = input_api.is_windows
     try:
         try:
-            subprocess.check_output(['gn', 'gen', outdir], shell=use_shell)
+            subprocess.check_output(
+                [sys.executable, 'third_party/depot_tools/gn.py', 'gen', outdir], shell=use_shell)
         except subprocess.CalledProcessError as e:
             return [
-                output_api.PresubmitError(
-                    'Unable to run gn gen for export_targets.py: %s' % e.output)
+                output_api.PresubmitError('Unable to run gn gen for export_targets.py: %s' %
+                                          e.output.decode())
             ]
         export_target_script = os.path.join(input_api.PresubmitLocalPath(), 'scripts',
                                             'export_targets.py')
@@ -311,11 +323,13 @@ def _CheckExportValidity(input_api, output_api):
                 shell=use_shell)
         except subprocess.CalledProcessError as e:
             if input_api.is_committing:
-                return [output_api.PresubmitError('export_targets.py failed: %s' % e.output)]
+                return [
+                    output_api.PresubmitError('export_targets.py failed: %s' % e.output.decode())
+                ]
             return [
                 output_api.PresubmitPromptWarning(
                     'export_targets.py failed, this may just be due to your local checkout: %s' %
-                    e.output)
+                    e.output.decode())
             ]
         return []
     finally:
@@ -323,7 +337,7 @@ def _CheckExportValidity(input_api, output_api):
 
 
 def _CheckTabsInSourceFiles(input_api, output_api):
-    """Forbids tab characters in source files due to a WebKit repo requirement. """
+    """Forbids tab characters in source files due to a WebKit repo requirement."""
 
     def implementation_and_headers_including_third_party(f):
         # Check third_party files too, because WebKit's checks don't make exceptions.
@@ -358,7 +372,7 @@ def is_ascii(s):
 
 
 def _CheckNonAsciiInSourceFiles(input_api, output_api):
-    """Forbids non-ascii characters in source files. """
+    """Forbids non-ascii characters in source files."""
 
     def implementation_and_headers(f):
         return input_api.FilterSourceFile(
@@ -382,10 +396,171 @@ def _CheckNonAsciiInSourceFiles(input_api, output_api):
     return []
 
 
+def _CheckCommentBeforeTestInTestFiles(input_api, output_api):
+    """Require a comment before TEST_P() and other tests."""
+
+    def test_files(f):
+        return input_api.FilterSourceFile(
+            f, files_to_check=(r'^src/tests/.+\.cpp$', r'^src/.+_unittest\.cpp$'))
+
+    tests_with_no_comment = []
+    for f in input_api.AffectedSourceFiles(test_files):
+        diff = f.GenerateScmDiff()
+        last_line_was_comment = False
+        for line in diff.splitlines():
+            # Skip removed lines
+            if line.startswith('-'):
+                continue
+
+            new_line_is_comment = line.startswith(' //') or line.startswith('+//')
+            new_line_is_test_declaration = (
+                line.startswith('+TEST_P(') or line.startswith('+TEST(') or
+                line.startswith('+TYPED_TEST('))
+
+            if new_line_is_test_declaration and not last_line_was_comment:
+                tests_with_no_comment.append(line[1:])
+
+            last_line_was_comment = new_line_is_comment
+
+    if tests_with_no_comment:
+        return [
+            output_api.PresubmitError(
+                'Tests without comment.',
+                items=sorted(tests_with_no_comment),
+                long_text='ANGLE requires a comment describing what a test does.')
+        ]
+    return []
+
+
+def _CheckWildcardInTestExpectationFiles(input_api, output_api):
+    """Require wildcard as API tag (i.e. in foo.bar/*) in expectations when no additional feature is
+    enabled."""
+
+    def expectation_files(f):
+        return input_api.FilterSourceFile(
+            f, files_to_check=[r'^src/tests/angle_end2end_tests_expectations.txt$'])
+
+    expectation_pattern = re.compile(r'^.*:\s*[a-zA-Z0-9._*]+\/([^ ]*)\s*=.*$')
+
+    expectations_without_wildcard = []
+    for f in input_api.AffectedSourceFiles(expectation_files):
+        diff = f.GenerateScmDiff()
+        for line in diff.splitlines():
+            # Only look at new lines
+            if not line.startswith('+'):
+                continue
+
+            match = re.match(expectation_pattern, line[1:].strip())
+            if match is None:
+                continue
+
+            tag = match.group(1)
+
+            # The tag is in the following general form:
+            #
+            #     FRONTENDAPI_BACKENDAPI[_FEATURE]*
+            #
+            # Any part of the above may be a wildcard.  Warn about usage of FRONTEND_BACKENDAPI as
+            # the tag.  Instead, the backend should be specified before the : and `*` used as the
+            # tag.  If any additional tags are present, it's a specific expectation that should
+            # remain specific (and not wildcarded).  NoFixture is an exception as X_Y_NoFixture is
+            # the generic form of the tags of tests that don't use the fixture.
+
+            sections = [section for section in tag.split('_') if section != 'NoFixture']
+
+            # Allow '*_...', or 'FRONTENDAPI_*_...'.
+            if '*' in sections[0] or (len(sections) > 1 and '*' in sections[1]):
+                continue
+
+            # Warn if no additional tags are present
+            if len(sections) == 2:
+                expectations_without_wildcard.append(line[1:])
+
+    if expectations_without_wildcard:
+        return [
+            output_api.PresubmitError(
+                'Use wildcard in API tags (after /) in angle_end2end_tests_expectations.txt.',
+                items=expectations_without_wildcard,
+                long_text="""ANGLE prefers end2end expections to use the following form:
+
+1234 MAC OPENGL : Foo.Bar/* = SKIP
+
+instead of:
+
+1234 MAC OPENGL : Foo.Bar/ES2_OpenGL = SKIP
+1234 MAC OPENGL : Foo.Bar/ES3_OpenGL = SKIP
+
+Expectatations that are specific (such as Foo.Bar/ES2_OpenGL_SomeFeature) are allowed.""")
+        ]
+    return []
+
+
+def _CheckShaderVersionInShaderLangHeader(input_api, output_api):
+    """Requires an update to ANGLE_SH_VERSION when ShaderLang.h or ShaderVars.h change."""
+
+    def headers(f):
+        return input_api.FilterSourceFile(
+            f,
+            files_to_check=(r'^include/GLSLANG/ShaderLang.h$', r'^include/GLSLANG/ShaderVars.h$'))
+
+    headers_changed = input_api.AffectedSourceFiles(headers)
+    if len(headers_changed) == 0:
+        return []
+
+    # Skip this check for reverts and rolls.  Unlike
+    # _CheckCommitMessageFormatting, relands are still checked because the
+    # original change might have incremented the version correctly, but the
+    # rebase over a new version could accidentally remove that (because another
+    # change in the meantime identically incremented it).
+    git_output = input_api.change.DescriptionText()
+    multiple_commits = _SplitIntoMultipleCommits(git_output)
+    for commit in multiple_commits:
+        if commit.startswith('Revert') or commit.startswith('Roll'):
+            return []
+
+    diffs = '\n'.join(f.GenerateScmDiff() for f in headers_changed)
+    versions = dict(re.findall(r'^([-+])#define ANGLE_SH_VERSION\s+(\d+)', diffs, re.M))
+
+    if len(versions) != 2 or int(versions['+']) <= int(versions['-']):
+        return [
+            output_api.PresubmitError(
+                'ANGLE_SH_VERSION should be incremented when ShaderLang.h or ShaderVars.h change.',
+            )
+        ]
+    return []
+
+
+def _CheckGClientExists(input_api, output_api, search_limit=None):
+    presubmit_path = pathlib.Path(input_api.PresubmitLocalPath())
+
+    for current_path in itertools.chain([presubmit_path], presubmit_path.parents):
+        gclient_path = current_path.joinpath('.gclient')
+        if gclient_path.exists() and gclient_path.is_file():
+            return []
+        # search_limit parameter is used in unit tests to prevent searching all the way to root
+        # directory for reproducibility.
+        elif search_limit != None and current_path == search_limit:
+            break
+
+    return [
+        output_api.PresubmitError(
+            'Missing .gclient file.',
+            long_text=textwrap.fill(
+                width=100,
+                text='The top level directory of the repository must contain a .gclient file.'
+                ' You can follow the steps outlined in the link below to get set up for ANGLE'
+                ' development:') +
+            '\n\nhttps://chromium.googlesource.com/angle/angle/+/refs/heads/main/doc/DevSetup.md')
+    ]
+
 def CheckChangeOnUpload(input_api, output_api):
     results = []
+    results.extend(input_api.canned_checks.CheckForCommitObjects(input_api, output_api))
     results.extend(_CheckTabsInSourceFiles(input_api, output_api))
     results.extend(_CheckNonAsciiInSourceFiles(input_api, output_api))
+    results.extend(_CheckCommentBeforeTestInTestFiles(input_api, output_api))
+    results.extend(_CheckWildcardInTestExpectationFiles(input_api, output_api))
+    results.extend(_CheckShaderVersionInShaderLangHeader(input_api, output_api))
     results.extend(_CheckCodeGeneration(input_api, output_api))
     results.extend(_CheckChangeHasBugField(input_api, output_api))
     results.extend(input_api.canned_checks.CheckChangeHasDescription(input_api, output_api))
@@ -395,6 +570,8 @@ def CheckChangeOnUpload(input_api, output_api):
         input_api.canned_checks.CheckPatchFormatted(
             input_api, output_api, result_factory=output_api.PresubmitError))
     results.extend(_CheckCommitMessageFormatting(input_api, output_api))
+    results.extend(_CheckGClientExists(input_api, output_api))
+
     return results
 
 
