@@ -12,9 +12,11 @@
 #define LIBANGLE_DISPLAY_H_
 
 #include <mutex>
-#include <set>
 #include <vector>
 
+#include "common/SimpleMutex.h"
+#include "common/WorkerThread.h"
+#include "common/platform.h"
 #include "libANGLE/AttributeMap.h"
 #include "libANGLE/BlobCache.h"
 #include "libANGLE/Caps.h"
@@ -24,10 +26,22 @@
 #include "libANGLE/Error.h"
 #include "libANGLE/LoggingAnnotator.h"
 #include "libANGLE/MemoryProgramCache.h"
+#include "libANGLE/MemoryShaderCache.h"
 #include "libANGLE/Observer.h"
+#include "libANGLE/ShareGroup.h"
 #include "libANGLE/Version.h"
 #include "platform/Feature.h"
-#include "platform/FrontendFeatures_autogen.h"
+#include "platform/autogen/FrontendFeatures_autogen.h"
+
+// Only DisplayCGL needs to be notified about an EGL call about to be made to prepare
+// per-thread data. Disable Display::prepareForCall on other platforms for performance.
+#if !defined(ANGLE_USE_DISPLAY_PREPARE_FOR_CALL)
+#    if ANGLE_ENABLE_CGL
+#        define ANGLE_USE_DISPLAY_PREPARE_FOR_CALL 1
+#    else
+#        define ANGLE_USE_DISPLAY_PREPARE_FOR_CALL 0
+#    endif
+#endif
 
 namespace angle
 {
@@ -45,7 +59,6 @@ namespace rx
 {
 class DisplayImpl;
 class EGLImplFactory;
-class ShareGroupImpl;
 }  // namespace rx
 
 namespace egl
@@ -57,64 +70,38 @@ class Surface;
 class Sync;
 class Thread;
 
-using ContextSet = std::set<gl::Context *>;
-using SurfaceSet = std::set<Surface *>;
+using SurfaceMap = angle::HashMap<GLuint, Surface *>;
+using ThreadSet  = angle::HashSet<Thread *>;
 
 struct DisplayState final : private angle::NonCopyable
 {
     DisplayState(EGLNativeDisplayType nativeDisplayId);
     ~DisplayState();
 
+    void notifyDeviceLost() const;
+
     EGLLabelKHR label;
-    ContextSet contextSet;
-    SurfaceSet surfaceSet;
-    std::vector<std::string> featureOverridesEnabled;
-    std::vector<std::string> featureOverridesDisabled;
-    bool featuresAllDisabled;
+    ContextMap contextMap;
+    mutable angle::SimpleMutex contextMapMutex;
+    SurfaceMap surfaceMap;
+    angle::FeatureOverrides featureOverrides;
     EGLNativeDisplayType displayId;
-};
 
-class ShareGroup final : angle::NonCopyable
-{
-  public:
-    ShareGroup(rx::EGLImplFactory *factory);
+    // Single-threaded and multithread pools for use by various parts of ANGLE, such as shader
+    // compilation.  These pools are internally synchronized.
+    std::shared_ptr<angle::WorkerThreadPool> singleThreadPool;
+    std::shared_ptr<angle::WorkerThreadPool> multiThreadPool;
 
-    void addRef();
-
-    void release(const egl::Display *display);
-
-    rx::ShareGroupImpl *getImplementation() const { return mImplementation; }
-
-    rx::Serial generateFramebufferSerial() { return mFramebufferSerialFactory.generate(); }
-
-    angle::FrameCaptureShared *getFrameCaptureShared() { return mFrameCaptureShared.get(); }
-
-    void finishAllContexts();
-
-    const ContextSet &getContexts() const { return mContexts; }
-    void addSharedContext(gl::Context *context);
-    void removeSharedContext(gl::Context *context);
-
-    size_t getShareGroupContextCount() const { return mContexts.size(); }
-
-  protected:
-    ~ShareGroup();
-
-  private:
-    size_t mRefCount;
-    rx::ShareGroupImpl *mImplementation;
-    rx::SerialFactory mFramebufferSerialFactory;
-
-    // Note: we use a raw pointer here so we can exclude frame capture sources from the build.
-    std::unique_ptr<angle::FrameCaptureShared> mFrameCaptureShared;
-
-    // The list of contexts within the share group
-    ContextSet mContexts;
+    mutable bool deviceLost;
 };
 
 // Constant coded here as a reasonable limit.
 constexpr EGLAttrib kProgramCacheSizeAbsoluteMax = 0x4000000;
 
+using ImageMap  = angle::HashMap<GLuint, Image *>;
+using StreamSet = angle::HashSet<Stream *>;
+using SyncMap   = angle::HashMap<GLuint, std::unique_ptr<Sync>>;
+
 class Display final : public LabeledObject,
                       public angle::ObserverInterface,
                       public angle::NonCopyable
@@ -134,17 +121,19 @@ class Display final : public LabeledObject,
     {
         Api,
         InternalCleanup,
-        ProcessExit,
 
         InvalidEnum,
         EnumCount = InvalidEnum,
     };
     Error terminate(Thread *thread, TerminateReason terminateReason);
-    Error destroyInvalidEglObjects();
+
+#if ANGLE_USE_DISPLAY_PREPARE_FOR_CALL
     // Called before all display state dependent EGL functions. Backends can set up, for example,
     // thread-specific backend state through this function. Not called for functions that do not
     // need the state.
     Error prepareForCall();
+#endif
+
     // Called on eglReleaseThread. Backends can tear down thread-specific backend state through
     // this function.
     Error releaseThread();
@@ -155,8 +144,7 @@ class Display final : public LabeledObject,
                                                 const AttributeMap &attribMap);
     static Display *GetExistingDisplayFromNativeDisplay(EGLNativeDisplayType nativeDisplay);
 
-    using EglDisplaySet = std::set<Display *>;
-    static EglDisplaySet GetEglDisplaySet();
+    using EglDisplaySet = angle::HashSet<Display *>;
 
     static const ClientExtensions &GetClientExtensions();
     static const std::string &GetClientExtensionString();
@@ -191,7 +179,6 @@ class Display final : public LabeledObject,
 
     Error createContext(const Config *configuration,
                         gl::Context *shareContext,
-                        const EGLenum clientType,
                         const AttributeMap &attribs,
                         gl::Context **outContext);
 
@@ -215,11 +202,11 @@ class Display final : public LabeledObject,
 
     bool isInitialized() const;
     bool isValidConfig(const Config *config) const;
-    bool isValidContext(const gl::Context *context) const;
-    bool isValidSurface(const Surface *surface) const;
-    bool isValidImage(const Image *image) const;
+    bool isValidContext(gl::ContextID contextID) const;
+    bool isValidSurface(SurfaceID surfaceID) const;
+    bool isValidImage(ImageID imageID) const;
     bool isValidStream(const Stream *stream) const;
-    bool isValidSync(const Sync *sync) const;
+    bool isValidSync(SyncID sync) const;
     bool isValidNativeWindow(EGLNativeWindowType window) const;
 
     Error validateClientBuffer(const Config *configuration,
@@ -230,7 +217,7 @@ class Display final : public LabeledObject,
                                     EGLenum target,
                                     EGLClientBuffer clientBuffer,
                                     const egl::AttributeMap &attribs) const;
-    Error valdiatePixmap(const Config *config,
+    Error validatePixmap(const Config *config,
                          EGLNativePixmapType pixmap,
                          const AttributeMap &attributes) const;
 
@@ -259,6 +246,7 @@ class Display final : public LabeledObject,
     const std::string &getExtensionString() const;
     const std::string &getVendorString() const;
     const std::string &getVersionString() const;
+    const std::string &getClientAPIString() const;
 
     std::string getBackendRendererDescription() const;
     std::string getBackendVendorString() const;
@@ -306,8 +294,15 @@ class Display final : public LabeledObject,
     egl::Error handleGPUSwitch();
     egl::Error forceGPUSwitch(EGLint gpuIDHigh, EGLint gpuIDLow);
 
-    std::mutex &getDisplayGlobalMutex() { return mDisplayGlobalMutex; }
-    std::mutex &getProgramCacheMutex() { return mProgramCacheMutex; }
+    egl::Error waitUntilWorkScheduled();
+
+    angle::SimpleMutex &getDisplayGlobalMutex() { return mDisplayGlobalMutex; }
+    angle::SimpleMutex &getProgramCacheMutex() { return mProgramCacheMutex; }
+
+    void lockVulkanQueue();
+    void unlockVulkanQueue();
+
+    gl::MemoryShaderCache *getMemoryShaderCache() { return &mMemoryShaderCache; }
 
     // Installs LoggingAnnotator as the global DebugAnnotator, for back-ends that do not implement
     // their own DebugAnnotator.
@@ -321,27 +316,69 @@ class Display final : public LabeledObject,
                                EGLBoolean *external_only,
                                EGLint *num_modifiers);
 
+    Error querySupportedCompressionRates(const Config *configuration,
+                                         const AttributeMap &attributes,
+                                         EGLint *rates,
+                                         EGLint rate_size,
+                                         EGLint *num_rates) const;
+
+    std::shared_ptr<angle::WorkerThreadPool> getSingleThreadPool() const
+    {
+        return mState.singleThreadPool;
+    }
+    std::shared_ptr<angle::WorkerThreadPool> getMultiThreadPool() const
+    {
+        return mState.multiThreadPool;
+    }
+
+    angle::ImageLoadContext getImageLoadContext() const;
+
+    const gl::Context *getContext(gl::ContextID contextID) const;
+    const egl::Surface *getSurface(egl::SurfaceID surfaceID) const;
+    const egl::Image *getImage(egl::ImageID imageID) const;
+    const egl::Sync *getSync(egl::SyncID syncID) const;
+    gl::Context *getContext(gl::ContextID contextID);
+    egl::Surface *getSurface(egl::SurfaceID surfaceID);
+    egl::Image *getImage(egl::ImageID imageID);
+    egl::Sync *getSync(egl::SyncID syncID);
+
+    const SyncMap &getSyncsForCapture() const { return mSyncMap; }
+    const ImageMap &getImagesForCapture() const { return mImageMap; }
+
+    // Initialize thread-local variables used by the Display and its backing implementations.  This
+    // includes:
+    //
+    // - The unlocked tail call to be run at the end of the entry point.
+    // - Scratch space for an egl::Error used by the backends (this is not used by all backends, and
+    //   access *must* be restricted to backends that use it).
+    //
+    static void InitTLS();
+    static angle::UnlockedTailCall *GetCurrentThreadUnlockedTailCall();
+    static Error *GetCurrentThreadErrorScratchSpace();
+
   private:
     Display(EGLenum platform, EGLNativeDisplayType displayId, Device *eglDevice);
 
     void setAttributes(const AttributeMap &attribMap) { mAttributeMap = attribMap; }
-
     void setupDisplayPlatform(rx::DisplayImpl *impl);
 
-    void updateAttribsFromEnvironment(const AttributeMap &attribMap);
-
     Error restoreLostDevice();
     Error releaseContext(gl::Context *context, Thread *thread);
+    Error releaseContextImpl(std::unique_ptr<gl::Context> &&context);
+    std::unique_ptr<gl::Context> eraseContextImpl(gl::Context *context, ContextMap *contexts);
 
     void initDisplayExtensions();
     void initVendorString();
     void initVersionString();
+    void initClientAPIString();
     void initializeFrontendFeatures();
 
     angle::ScratchBuffer requestScratchBufferImpl(std::vector<angle::ScratchBuffer> *bufferVector);
     void returnScratchBufferImpl(angle::ScratchBuffer scratchBuffer,
                                  std::vector<angle::ScratchBuffer> *bufferVector);
 
+    Error destroyInvalidEglObjects();
+
     DisplayState mState;
     rx::DisplayImpl *mImplementation;
     angle::ObserverBinding mGPUSwitchedBinding;
@@ -350,28 +387,27 @@ class Display final : public LabeledObject,
 
     ConfigSet mConfigSet;
 
-    typedef std::set<Image *> ImageSet;
-    ImageSet mImageSet;
-
-    typedef std::set<Stream *> StreamSet;
+    ImageMap mImageMap;
     StreamSet mStreamSet;
 
-    typedef std::set<Sync *> SyncSet;
-    SyncSet mSyncSet;
+    SyncMap mSyncMap;
 
-    void destroyImageImpl(Image *image, ImageSet *images);
+    static constexpr size_t kMaxSyncPoolSizePerType = 32;
+    using SyncPool = angle::FixedVector<std::unique_ptr<Sync>, kMaxSyncPoolSizePerType>;
+    std::map<EGLenum, SyncPool> mSyncPools;
+
+    void destroyImageImpl(Image *image, ImageMap *images);
     void destroyStreamImpl(Stream *stream, StreamSet *streams);
-    Error destroySurfaceImpl(Surface *surface, SurfaceSet *surfaces);
-    void destroySyncImpl(Sync *sync, SyncSet *syncs);
+    Error destroySurfaceImpl(Surface *surface, SurfaceMap *surfaces);
+    void destroySyncImpl(SyncID syncId, SyncMap *syncs);
 
-    std::mutex mInvalidEglObjectsMutex;
-    ImageSet mInvalidImageSet;
+    ContextMap mInvalidContextMap;
+    ImageMap mInvalidImageMap;
     StreamSet mInvalidStreamSet;
-    SurfaceSet mInvalidSurfaceSet;
-    SyncSet mInvalidSyncSet;
+    SurfaceMap mInvalidSurfaceMap;
+    SyncMap mInvalidSyncMap;
 
     bool mInitialized;
-    bool mDeviceLost;
 
     Caps mCaps;
 
@@ -380,31 +416,40 @@ class Display final : public LabeledObject,
 
     std::string mVendorString;
     std::string mVersionString;
+    std::string mClientAPIString;
 
     Device *mDevice;
     Surface *mSurface;
     EGLenum mPlatform;
     angle::LoggingAnnotator mAnnotator;
 
+    // mManagersMutex protects mTextureManager and mSemaphoreManager
+    ContextMutex *mManagersMutex;
     gl::TextureManager *mTextureManager;
     gl::SemaphoreManager *mSemaphoreManager;
+
     BlobCache mBlobCache;
     gl::MemoryProgramCache mMemoryProgramCache;
+    gl::MemoryShaderCache mMemoryShaderCache;
     size_t mGlobalTextureShareGroupUsers;
     size_t mGlobalSemaphoreShareGroupUsers;
 
+    gl::HandleAllocator mImageHandleAllocator;
+    gl::HandleAllocator mSurfaceHandleAllocator;
+    gl::HandleAllocator mSyncHandleAllocator;
+
     angle::FrontendFeatures mFrontendFeatures;
 
     angle::FeatureList mFeatures;
 
-    std::mutex mScratchBufferMutex;
+    angle::SimpleMutex mScratchBufferMutex;
     std::vector<angle::ScratchBuffer> mScratchBuffers;
     std::vector<angle::ScratchBuffer> mZeroFilledBuffers;
 
-    std::mutex mDisplayGlobalMutex;
-    std::mutex mProgramCacheMutex;
+    angle::SimpleMutex mDisplayGlobalMutex;
+    angle::SimpleMutex mProgramCacheMutex;
 
-    bool mIsTerminated;
+    bool mTerminatedByApi;
 };
 
 }  // namespace egl
