@@ -43,6 +43,8 @@ ContextMenuContext::ContextMenuContext(Type type, const HitTestResult& hitTestRe
     , m_hitTestResult(hitTestResult)
     , m_event(WTFMove(event))
     , m_hasEntireImage(hitTestResult.hasEntireImage())
+    , m_allowsFollowingLink(hitTestResult.allowsFollowingLink())
+    , m_allowsFollowingImageURL(hitTestResult.allowsFollowingImageURL())
 {
 }
 
