@@ -44,11 +44,15 @@
 #include "pas_status_reporter.h"
 #include "pas_thread_local_cache.h"
 #include "pas_utility_heap.h"
+#include "pas_utils.h"
 #include <stdio.h>
+#if !PAS_OS(WINDOWS)
 #include <sys/time.h>
 #include <unistd.h>
+#endif
 
 static const bool verbose = false;
+static bool is_shut_down_enabled = true;
 
 bool pas_scavenger_is_enabled = true;
 bool pas_scavenger_eligibility_notification_has_been_deferred = false;
@@ -69,8 +73,17 @@ double pas_scavenger_period_in_milliseconds = 100.;
 uint64_t pas_scavenger_max_epoch_delta = 300ll * 1000ll * 1000ll;
 #endif
 
+static uint32_t pas_scavenger_tick_count = 0;
+/* Run thread-local-cache decommit once a N. It should be power of two. */
+#define PAS_THREAD_LOCAL_CACHE_DECOMMIT_PERIOD_COUNT 128 /* Roughly speaking, it runs once per 13 seconds. */
+
 #if PAS_OS(DARWIN)
-qos_class_t pas_scavenger_requested_qos_class = QOS_CLASS_USER_INITIATED;
+static _Atomic qos_class_t pas_scavenger_requested_qos_class = QOS_CLASS_USER_INITIATED;
+
+void pas_scavenger_set_requested_qos_class(qos_class_t qos_class)
+{
+    pas_scavenger_requested_qos_class = qos_class;
+}
 #endif
 
 pas_scavenger_activity_callback pas_scavenger_did_start_callback = NULL;
@@ -98,6 +111,7 @@ static pas_scavenger_data* ensure_data_instance(pas_lock_hold_mode heap_lock_hol
         
         pthread_mutex_init(&instance->lock, NULL);
         pthread_cond_init(&instance->cond, NULL);
+        pthread_mutex_init(&instance->foreign_work.lock, NULL);
 
         pas_fence();
         
@@ -110,11 +124,19 @@ static pas_scavenger_data* ensure_data_instance(pas_lock_hold_mode heap_lock_hol
 
 static double get_time_in_milliseconds(void)
 {
+#if PAS_OS(WINDOWS)
+    LARGE_INTEGER frequency, counter;
+    QueryPerformanceFrequency(&frequency);
+    QueryPerformanceCounter(&counter);
+
+    return (counter.QuadPart * 1000.) / frequency.QuadPart;
+#else
     struct timeval current_time;
     
     gettimeofday(&current_time, NULL);
     
     return current_time.tv_sec * 1000. + current_time.tv_usec / 1000.;
+#endif
 }
 
 static void timed_wait(pthread_cond_t* cond, pthread_mutex_t* mutex,
@@ -129,12 +151,12 @@ static void timed_wait(pthread_cond_t* cond, pthread_mutex_t* mutex,
         (uint64_t)(1000. * 1000. * 1000.));
     
     if (verbose) {
-        printf("Doing timed wait with target wake up at %.2lf.\n",
+        pas_log("Doing timed wait with target wake up at %.2lf.\n",
                absolute_timeout_in_milliseconds);
     }
     pthread_cond_timedwait(cond, mutex, &time_to_wake_up);
     if (verbose)
-        printf("Woke up from timed wait at %.2lf.\n", get_time_in_milliseconds());
+        pas_log("Woke up from timed wait at %.2lf.\n", get_time_in_milliseconds());
 }
 
 static bool handle_expendable_memory(pas_expendable_memory_scavenge_kind kind)
@@ -147,10 +169,21 @@ static bool handle_expendable_memory(pas_expendable_memory_scavenge_kind kind)
     return should_go_again;
 }
 
+#if PAS_OS(WINDOWS)
+#define R_NULL 0
+
+unsigned scavenger_thread_main(void* arg)
+#else
+#define R_NULL NULL
+
 static void* scavenger_thread_main(void* arg)
+#endif
 {
     pas_scavenger_data* data;
     pas_scavenger_activity_callback did_start_callback;
+#if PAS_OS(DARWIN)
+    qos_class_t configured_qos_class;
+#endif
     
     PAS_UNUSED_PARAM(arg);
     
@@ -173,26 +206,43 @@ static void* scavenger_thread_main(void* arg)
     
     data = ensure_data_instance(pas_lock_is_not_held);
     
+#if PAS_OS(DARWIN)
+    configured_qos_class = pas_scavenger_requested_qos_class;
+    pthread_set_qos_class_self_np(configured_qos_class, 0);
+#endif
+
+    PAS_PROFILE(SCAVENGER_THREAD_MAIN, data);
+
     for (;;) {
         pas_page_sharing_pool_scavenge_result scavenge_result;
         bool should_shut_down;
         double time_in_milliseconds;
         double absolute_timeout_in_milliseconds_for_period_sleep;
         pas_scavenger_activity_callback completion_callback;
+        pas_thread_local_cache_decommit_action thread_local_cache_decommit_action;
         bool should_go_again;
         uint64_t epoch;
         uint64_t delta;
         uint64_t max_epoch;
+        int installed_foreign_work_descriptors;
         bool did_overflow;
+#if PAS_OS(DARWIN)
+        qos_class_t current_qos_class;
+#endif
 
 #if PAS_OS(DARWIN)
-        pthread_set_qos_class_self_np(pas_scavenger_requested_qos_class, 0);
+        current_qos_class = pas_scavenger_requested_qos_class;
+        if (configured_qos_class != current_qos_class) {
+            configured_qos_class = current_qos_class;
+            pthread_set_qos_class_self_np(configured_qos_class, 0);
+        }
 #endif
+        ++pas_scavenger_tick_count;
 
         should_go_again = false;
         
         if (verbose)
-            printf("Scavenger is running.\n");
+            pas_log("Scavenger is running.\n");
 
 #if PAS_LOCAL_ALLOCATOR_MEASURE_REFILL_EFFICIENCY
         pas_local_allocator_refill_efficiency_lock_lock();
@@ -209,9 +259,16 @@ static void* scavenger_thread_main(void* arg)
             pas_utility_heap_for_all_allocators(pas_allocator_scavenge_request_stop_action,
                                                 pas_lock_is_not_held);
         
+        thread_local_cache_decommit_action = pas_thread_local_cache_decommit_no_action;
+        if ((pas_scavenger_tick_count % PAS_THREAD_LOCAL_CACHE_DECOMMIT_PERIOD_COUNT) == 0) {
+            if (verbose)
+                pas_log("Attempt to decommit unused TLC\n");
+            thread_local_cache_decommit_action = pas_thread_local_cache_decommit_if_possible_action;
+        }
         should_go_again |=
             pas_thread_local_cache_for_all(pas_allocator_scavenge_request_stop_action,
-                                           pas_deallocator_scavenge_flush_log_if_clean_action);
+                                           pas_deallocator_scavenge_flush_log_if_clean_action,
+                                           thread_local_cache_decommit_action);
 
         should_go_again |= handle_expendable_memory(pas_expendable_memory_scavenge_periodic);
 
@@ -252,13 +309,24 @@ static void* scavenger_thread_main(void* arg)
             break;
         } }
 
+
+        installed_foreign_work_descriptors = data->foreign_work.next_open_descriptor;
+        PAS_ASSERT(installed_foreign_work_descriptors <= PAS_SCAVENGER_MAX_FOREIGN_WORK_DESCRIPTORS);
+        pas_fence();
+        for (int i = 0; i < installed_foreign_work_descriptors; i++) {
+            void* userdata = data->foreign_work.descriptors[i].userdata;
+            uint32_t requested_period_ticks = 1 << data->foreign_work.descriptors[i].period_log2_ticks;
+            if (!requested_period_ticks || pas_scavenger_tick_count % requested_period_ticks == 0)
+                should_go_again |= data->foreign_work.descriptors[i].func(userdata);
+        }
+
         if (verbose) {
             pas_log("%d: %.0lf: scavenger freed %zu bytes (%s, should_go_again = %s).\n",
                     getpid(), get_time_in_milliseconds(), scavenge_result.total_bytes,
                     pas_page_sharing_pool_take_result_get_string(scavenge_result.take_result),
                     should_go_again ? "yes" : "no");
         }
-        
+
         completion_callback = pas_scavenger_completion_callback;
         if (completion_callback)
             completion_callback();
@@ -273,7 +341,7 @@ static void* scavenger_thread_main(void* arg)
         time_in_milliseconds = get_time_in_milliseconds();
         
         if (verbose)
-            printf("Finished a round of scavenging at %.2lf.\n", time_in_milliseconds);
+            pas_log("Finished a round of scavenging at %.2lf.\n", time_in_milliseconds);
         
         /* By default we need to sleep for a short while and then try again. */
         absolute_timeout_in_milliseconds_for_period_sleep =
@@ -281,25 +349,25 @@ static void* scavenger_thread_main(void* arg)
 
         if (should_go_again) {
             if (verbose)
-                printf("Waiting for a period.\n");
+                pas_log("Waiting for a period.\n");
 
             /* This field is accessed a lot by other threads, so don't write to it if we don't
                have to. */
             if (pas_scavenger_current_state != pas_scavenger_state_polling)
                 pas_scavenger_current_state = pas_scavenger_state_polling;
-        } else {
+        } else if (PAS_LIKELY(is_shut_down_enabled)) {
             double absolute_timeout_in_milliseconds_for_deep_pre_sleep;
             
             if (pas_scavenger_current_state == pas_scavenger_state_polling) {
                 if (verbose)
-                    printf("Will consider deep sleep.\n");
+                    pas_log("Will consider deep sleep.\n");
                 
                 /* do one more round of polling but this time indicating that it's the last
                    chance. */
                 pas_scavenger_current_state = pas_scavenger_state_deep_sleep;
             } else {
                 if (verbose)
-                    printf("Considering deep sleep.\n");
+                    pas_log("Considering deep sleep.\n");
                 
                 PAS_ASSERT(pas_scavenger_current_state == pas_scavenger_state_deep_sleep);
                 
@@ -319,19 +387,21 @@ static void* scavenger_thread_main(void* arg)
             }
         }
         
-        while (get_time_in_milliseconds() < absolute_timeout_in_milliseconds_for_period_sleep
-               && !pas_scavenger_should_suspend_count) {
-            timed_wait(&data->cond, &data->lock,
-                       absolute_timeout_in_milliseconds_for_period_sleep);
+        if (PAS_LIKELY(is_shut_down_enabled)) {
+            while (get_time_in_milliseconds() < absolute_timeout_in_milliseconds_for_period_sleep
+                   && !pas_scavenger_should_suspend_count) {
+                timed_wait(&data->cond, &data->lock,
+                           absolute_timeout_in_milliseconds_for_period_sleep);
+            }
+
+            should_shut_down |= !!pas_scavenger_should_suspend_count;
+
+            if (should_shut_down) {
+                pas_scavenger_current_state = pas_scavenger_state_no_thread;
+                pthread_cond_broadcast(&data->cond);
+            }
         }
 
-        should_shut_down |= !!pas_scavenger_should_suspend_count;
-        
-        if (should_shut_down) {
-            pas_scavenger_current_state = pas_scavenger_state_no_thread;
-            pthread_cond_broadcast(&data->cond);
-        }
-        
         pthread_mutex_unlock(&data->lock);
         
         if (should_shut_down) {
@@ -342,13 +412,43 @@ static void* scavenger_thread_main(void* arg)
                 shut_down_callback();
             
             if (verbose)
-                printf("Killing the scavenger.\n");
-            return NULL;
+                pas_log("Killing the scavenger.\n");
+            return R_NULL;
         }
     }
 
-    PAS_ASSERT(!"Should not be reached");
-    return NULL;
+    PAS_ASSERT_NOT_REACHED();
+    return R_NULL;
+}
+
+bool pas_scavenger_try_install_foreign_work_callback(
+    pas_scavenger_foreign_work_callback callback,
+    uint32_t period_log2_ms,
+    void* userdata)
+{
+    pas_scavenger_data* data;
+
+    PAS_ASSERT(callback);
+
+    data = ensure_data_instance(pas_lock_is_not_held);
+    pthread_mutex_lock(&data->foreign_work.lock);
+
+    int slot = data->foreign_work.next_open_descriptor;
+    if (slot >= PAS_SCAVENGER_MAX_FOREIGN_WORK_DESCRIPTORS)
+        return false;
+
+    double requested_period_ms = pow(2.0, period_log2_ms);
+    uint32_t requested_ticks = (uint32_t)(requested_period_ms / pas_scavenger_period_in_milliseconds);
+
+    data->foreign_work.descriptors[slot].period_log2_ticks = pas_log2(requested_ticks);
+    data->foreign_work.descriptors[slot].func = callback;
+    data->foreign_work.descriptors[slot].userdata = userdata;
+    pas_store_store_fence();
+    data->foreign_work.next_open_descriptor = slot + 1;
+
+    pthread_mutex_unlock(&data->foreign_work.lock);
+
+    return true;
 }
 
 bool pas_scavenger_did_create_eligible(void)
@@ -394,7 +494,7 @@ void pas_scavenger_notify_eligibility_if_needed(void)
         return;
     
     if (verbose)
-        printf("It's not polling so need to do something.\n");
+        pas_log("It's not polling so need to do something.\n");
     
     data = ensure_data_instance(pas_lock_is_not_held);
     pthread_mutex_lock(&data->lock);
@@ -482,7 +582,8 @@ void pas_scavenger_clear_all_caches(void)
     pas_scavenger_clear_all_caches_except_remote_tlcs();
     
     pas_thread_local_cache_for_all(pas_allocator_scavenge_force_stop_action,
-                                   pas_deallocator_scavenge_flush_log_action);
+                                   pas_deallocator_scavenge_flush_log_action,
+                                   pas_thread_local_cache_decommit_if_possible_action);
 }
 
 void pas_scavenger_decommit_expendable_memory(void)
@@ -518,7 +619,7 @@ void pas_scavenger_perform_synchronous_operation(
 {
     switch (kind) {
     case pas_scavenger_invalid_synchronous_operation_kind:
-        PAS_ASSERT(!"Should not be reached");
+        PAS_ASSERT_NOT_REACHED();
         return;
     case pas_scavenger_clear_all_non_tlc_caches_kind:
         pas_scavenger_clear_all_non_tlc_caches();
@@ -539,7 +640,14 @@ void pas_scavenger_perform_synchronous_operation(
         pas_scavenger_run_synchronously_now();
         return;
     }
-    PAS_ASSERT(!"Should not be reached");
+    PAS_ASSERT_NOT_REACHED();
+}
+
+void pas_scavenger_disable_shut_down(void)
+{
+    pas_scavenger_suspend();
+    is_shut_down_enabled = false;
+    pas_scavenger_resume();
 }
 
 #endif /* LIBPAS_ENABLED */
